@@ -1,5 +1,6 @@
 #!/usr/bin/python3
 
+import logging
 import os
 import re
 import sys
@@ -28,11 +29,20 @@ from qgispluginci.exceptions import (
     BuiltResourceInSources,
     GithubReleaseCouldNotUploadAsset,
     GithubReleaseNotFound,
+    MissingChangelog,
     UncommitedChanges,
 )
 from qgispluginci.parameters import Parameters
 from qgispluginci.translation import Translation
-from qgispluginci.utils import configure_file, parse_tag, replace_in_file
+from qgispluginci.utils import (
+    configure_file,
+    convert_octets,
+    parse_tag,
+    replace_in_file,
+)
+
+# GLOBALS
+logger = logging.getLogger(__name__)
 
 
 def create_archive(
@@ -54,13 +64,16 @@ def create_archive(
     initial_stash = None
     diff = repo.index.diff(None)
     if diff:
-        print("Uncommitted changes:")
+        logger.info("There are uncommitted changes:")
         for diff in diff:
-            print(diff)
+            logger.info(diff)
         if not allow_uncommitted_changes:
-            raise UncommitedChanges(
-                "You have uncommitted changes. Stash or commit them or use --allow-uncommitted-changes."
+            err_msg = (
+                "You have uncommitted changes. "
+                "Stash or commit them or use -c / --allow-uncommitted-changes option."
             )
+            logger.error(err_msg, exc_info=UncommitedChanges())
+            sys.exit(1)
         else:
             initial_stash = repo.git.stash("create")
 
@@ -81,17 +94,16 @@ def create_archive(
                         r"^changelog=.*$",
                         "changelog={}".format(content),
                     )
-            except Exception as e:
+            except Exception as exc:
                 # Do not fail the release process if something is wrong when parsing the changelog
                 replace_in_file(
                     "{}/metadata.txt".format(parameters.plugin_path),
                     r"^changelog=.*$",
                     "",
                 )
-                print(
-                    "An exception occurred while parsing the changelog file : {}".format(
-                        e
-                    )
+                logger.warning(
+                    f"An exception occurred while parsing the changelog file: {exc}",
+                    exc_info=exc,
                 )
     else:
         # Remove the changelog line
@@ -158,29 +170,26 @@ def create_archive(
     if stash == "" or stash is None:
         stash = "HEAD"
     # create TAR archive
-    print("archive plugin with stash: {}".format(stash))
+    logger.debug(f"Git archive plugin with stash: {stash}")
     repo.git.archive(stash, "-o", top_tar_file, parameters.plugin_path)
     # adding submodules
     for submodule in repo.submodules:
         _, sub_tar_file = mkstemp(suffix=".tar")
         if submodule.path.split("/")[0] != parameters.plugin_path:
-            print(
-                "skipping submodule not in plugin source directory ({})".format(
-                    submodule.name
-                )
+            logger.debug(
+                f"Skipping submodule not in plugin source directory: {submodule.name}"
             )
             continue
         if not disable_submodule_update:
             submodule.update(init=True)
         sub_repo = submodule.module()
-        print("archive submodule:", sub_repo)
+        logger.info("Git archive submodule: {sub_repo}")
         sub_repo.git.archive(
             "HEAD", "--prefix", "{}/".format(submodule.path), "-o", sub_tar_file
         )
         with tarfile.open(top_tar_file, mode="a") as tt:
             with tarfile.open(sub_tar_file, mode="r:") as st:
                 for m in st.getmembers():
-                    # print('adding', m, m.type, m.isfile())
                     if not m.isfile():
                         continue
                     tt.add(m.name)
@@ -188,34 +197,38 @@ def create_archive(
     # add translation files
     if add_translations:
         with tarfile.open(top_tar_file, mode="a") as tt:
-            print("adding translations")
+            logger.debug("Adding translations")
             for file in glob("{}/i18n/*.qm".format(parameters.plugin_path)):
-                print("  adding translation: {}".format(os.path.basename(file)))
+                logger.debug("  adding translation: {}".format(os.path.basename(file)))
                 # https://stackoverflow.com/a/48462950/1548052
                 tt.add(file)
 
     # compile qrc files
-    pyqt5ac.main(
-        ioPaths=[
-            [
-                "{}/*.qrc".format(parameters.plugin_path),
-                "{}/%%FILENAME%%_rc.py".format(parameters.plugin_path),
+    if list(Path(parameters.plugin_path).glob("*.qrc")):
+        pyqt5ac.main(
+            ioPaths=[
+                [
+                    "{}/*.qrc".format(parameters.plugin_path),
+                    "{}/%%FILENAME%%_rc.py".format(parameters.plugin_path),
+                ]
             ]
-        ]
-    )
-    for file in glob("{}/*_rc.py".format(parameters.plugin_path)):
-        with tarfile.open(top_tar_file, mode="r:") as tt:
-            for n in tt.getnames():
-                if n == file:
-                    raise BuiltResourceInSources(
-                        'The file "{}" is present in the sources and its name conflicts with a just built resource. You might want to remove it from the sources or setting export-ignore in .gitattributes config file.'.format(
-                            file
+        )
+        for file in glob("{}/*_rc.py".format(parameters.plugin_path)):
+            with tarfile.open(top_tar_file, mode="r:") as tt:
+                for n in tt.getnames():
+                    if n == file:
+                        err_msg = (
+                            f"The file {file} is present in the sources and its name "
+                            "conflicts with a just built resource. "
+                            "You might want to remove it from the sources or "
+                            "setting export-ignore in .gitattributes config file."
                         )
-                    )
-        with tarfile.open(top_tar_file, mode="a") as tt:
-            print("  adding resource: {}".format(file))
-            # https://stackoverflow.com/a/48462950/1548052
-            tt.add(file)
+                        logger.error(err_msg, exc_info=BuiltResourceInSources())
+                        sys.exit(1)
+            with tarfile.open(top_tar_file, mode="a") as tt:
+                logger.debug("\tAdding resource: {}".format(file))
+                # https://stackoverflow.com/a/48462950/1548052
+                tt.add(file)
 
     # converting to ZIP
     # why using TAR before? because it provides the prefix and makes things easier
@@ -238,14 +251,15 @@ def create_archive(
                 # code : https://github.com/python/cpython/blob/b885b8f4be9c74ef1ce7923dbf055c31e7f47735/Lib/zipfile.py#L545
                 # see https://stackoverflow.com/questions/434641/how-do-i-set-permissions-attributes-on-a-file-in-a-zip-file-using-pythons-zip/53008127#53008127
                 info.external_attr = (st[0] & 0xFFFF) << 16  # Unix attributes
+                info.compress_type = zf.compression
                 zf.writestr(info, fl)
 
-    print("-------")
-    print("files in ZIP archive ({}):".format(archive_name))
+    logger.debug("-" * 40)
+    logger.debug("Files in ZIP archive ({}):".format(archive_name))
     with zipfile.ZipFile(file=archive_name, mode="r") as zf:
         for f in zf.namelist():
-            print(f)
-    print("-------")
+            logger.debug(f)
+    logger.debug("-" * 40)
 
     # checkout to reset changes
     if initial_stash:
@@ -254,6 +268,12 @@ def create_archive(
     else:
         repo.git.checkout("--", ".")
 
+    # print the result
+    print(  # noqa: T2
+        f"Plugin archive created: {archive_name} "
+        f"({convert_octets(Path(archive_name).stat().st_size)})"
+    )
+
 
 def upload_asset_to_github_release(
     parameters: Parameters,
@@ -266,32 +286,41 @@ def upload_asset_to_github_release(
     slug = "{}/{}".format(parameters.github_organization_slug, parameters.project_slug)
     repo = Github(github_token).get_repo(slug)
     try:
-        print(
-            "Getting release on {}/{}".format(
-                parameters.github_organization_slug, parameters.project_slug
-            )
+        logger.debug(
+            f"Getting release on {parameters.github_organization_slug}"
+            f"/{parameters.project_slug}"
         )
         gh_release = repo.get_release(id=release_tag)
-        print(gh_release, gh_release.tag_name, gh_release.upload_url)
-    except GithubException as e:
-        raise GithubReleaseNotFound("Release {} not found".format(release_tag))
+        logger.debug(
+            f"Release retrieved from GitHub: {gh_release}, "
+            f"{gh_release.tag_name}, "
+            f"{gh_release.upload_url}"
+        )
+    except GithubException as exc:
+        logger.error(
+            f"Release {release_tag} not found for {slug}",
+            exc_info=GithubReleaseNotFound(exc),
+        )
+        sys.exit(1)
     try:
         assert os.path.exists(asset_path)
         if asset_name:
-            print("Uploading asset: {} as {}".format(asset_path, asset_name))
-            gh_release.upload_asset(path=asset_path, label=asset_name, name=asset_name)
-        else:
-            print("Uploading asset: {}".format(asset_path))
-            gh_release.upload_asset(asset_path)
-        print("OK")
-    except GithubException as e:
-        print(e)
-        raise GithubReleaseCouldNotUploadAsset(
-            "Could not upload asset for release {}. "
-            "Are you sure the user for the given token can upload asset to this repo?".format(
-                release_tag
+            logger.debug(f"Uploading asset: {asset_path} as {asset_name}")
+
+            uploaded_asset = gh_release.upload_asset(
+                path=asset_path, label=asset_name, name=asset_name
             )
+        else:
+            logger.debug(f"Uploading asset: {asset_path}")
+            uploaded_asset = gh_release.upload_asset(asset_path)
+        logger.info(f"Asset successfully uploaded: {uploaded_asset.url}")
+    except GithubException as exc:
+        logger.error(
+            f"Could not upload asset for release {release_tag} on {slug}. "
+            "Are you sure the user for the given token can upload asset to this repo?",
+            exc_info=GithubReleaseCouldNotUploadAsset(exc),
         )
+        sys.exit(1)
 
 
 def release_is_prerelease(
@@ -311,15 +340,22 @@ def release_is_prerelease(
     slug = "{}/{}".format(parameters.github_organization_slug, parameters.project_slug)
     repo = Github(github_token).get_repo(slug)
     try:
-        print(
-            "Getting release on {}/{}".format(
-                parameters.github_organization_slug, parameters.project_slug
-            )
+        logger.debug(
+            f"Getting release on {parameters.github_organization_slug}"
+            f"/{parameters.project_slug}"
         )
         gh_release = repo.get_release(id=release_tag)
-        print(gh_release, gh_release.tag_name, gh_release.upload_url)
-    except GithubException as e:
-        raise GithubReleaseNotFound("Release {} not found".format(release_tag))
+        logger.debug(
+            f"Release retrieved from GitHub: {gh_release}, "
+            f"{gh_release.tag_name}, "
+            f"{gh_release.upload_url}"
+        )
+    except GithubException as exc:
+        logger.error(
+            f"Release {release_tag} not found. Trace: {exc}",
+            exc_info=GithubReleaseNotFound(),
+        )
+        sys.exit(1)
     return gh_release.prerelease
 
 
@@ -327,6 +363,7 @@ def create_plugin_repo(
     parameters: Parameters,
     release_version: str,
     release_tag: str,
+    is_prerelease: bool,
     archive: str,
     osgeo_username,
     plugin_repo_url=None,
@@ -345,7 +382,7 @@ def create_plugin_repo(
         "__PLUGINZIP__": archive,
         "__OSGEO_USERNAME__": osgeo_username or parameters.author,
         "__DEPRECATED__": str(parameters.deprecated),
-        "__EXPERIMENTAL__": str(parameters.experimental),
+        "__EXPERIMENTAL__": str(is_prerelease or parameters.experimental),
         "__TAGS__": parameters.tags,
         "__ICON__": parameters.icon,
         "__AUTHOR__": parameters.author,
@@ -390,34 +427,39 @@ def upload_plugin_to_osgeo(username: str, password: str, archive: str):
     archive
         The plugin archive file path to be uploaded
     """
-    address = "https://{username}:{password}@plugins.qgis.org:443/plugins/RPC2/".format(
-        username=username, password=password
-    )
+    address = f"https://{username}:{password}@plugins.qgis.org:443/plugins/RPC2/"
 
     server = xmlrpc.client.ServerProxy(address, verbose=True)
 
     try:
+        logger.debug(f"Start uploading {archive} to QGIS plugins repository.")
         with open(archive, "rb") as handle:
             plugin_id, version_id = server.plugin.upload(
                 xmlrpc.client.Binary(handle.read())
             )
-        print(f"Plugin ID: {plugin_id}")
-        print(f"Version ID: {version_id}")
+        logger.debug(f"Plugin ID: {plugin_id}")
+        logger.debug(f"Version ID: {version_id}")
     except xmlrpc.client.ProtocolError as err:
-        print("A protocol error occurred")
         url = re.sub(r":[^/].*@", ":******@", err.url)
-        print(f"URL: {url}")
-        print(f"HTTP/HTTPS headers: {err.headers}")
-        print(f"Error code: {err.errcode}")
-        print(f"Error message: {err.errmsg}")
-        print(f"Plugin path : {archive}")
-        sys.exit(1)
+        err_msg = (
+            "=== A protocol error occurred ===\n"
+            f"URL: {url}\n"
+            f"HTTP/HTTPS headers: {err.headers}\n"
+            f"Error code: {err.errcode}\n"
+            f"Error message: {err.errmsg}\n"
+            f"Plugin path: {archive}"
+        )
+        logger.error(err_msg)
+        sys.exit(err_msg)
     except xmlrpc.client.Fault as err:
-        print("A fault occurred")
-        print(f"Fault code: {err.faultCode}")
-        print(f"Fault string: {err.faultString}")
-        print(f"Plugin path : {archive}")
-        sys.exit(1)
+        err_msg = (
+            "=== A fault occurred occurred ===\n"
+            f"Fault code: {err.faultCode}\n"
+            f"Fault string: {err.faultString}\n"
+            f"Plugin path: {archive}"
+        )
+        logger.error(err_msg)
+        sys.exit(err_msg)
 
 
 def release(
@@ -468,7 +510,10 @@ def release(
             parent_folder=Path(parameters.plugin_path).resolve().parent,
             changelog_path=parameters.changelog_path,
         )
-        release_version = parser.latest_version()
+        if parser.has_changelog():
+            release_version = parser.latest_version()
+        else:
+            release_version = "latest"
 
     release_tag = release_tag or release_version
 
@@ -481,10 +526,15 @@ def release(
 
     archive_name = parameters.archive_name(parameters.plugin_path, release_version)
 
+    # check if the GitHub release is a regular or pre-release
     is_prerelease = release_is_prerelease(
         parameters, release_tag=release_tag, github_token=github_token
     )
-    print("*** is pre-release: {}".format("YES" if is_prerelease else "NO"))
+
+    if is_prerelease:
+        logger.info(f"{release_tag} is a pre-release.")
+    else:
+        logger.info(f"{release_tag} is a regular release.")
 
     create_archive(
         parameters,
@@ -526,6 +576,7 @@ def release(
                 parameters=parameters,
                 release_version=release_version,
                 release_tag=release_tag,
+                is_prerelease=is_prerelease,
                 archive=archive_name,
                 osgeo_username=osgeo_username,
             )
@@ -546,7 +597,7 @@ def release(
             osgeo_username=osgeo_username,
             plugin_repo_url=plugin_repo_url,
         )
-        print("Local XML repo file created : {}".format(xml_repo))
+        logger.info(f"Local XML repo file created : {xml_repo}")
 
     if osgeo_username is not None:
         assert osgeo_password is not None
